pub trait BitmapData {
    fn data(&self) -> &[u8];
    fn width(&self) -> usize;
    fn height(&self) -> usize;
    fn stride(&self) -> usize;
}

pub trait BitmapDataMut: BitmapData {
    fn data_mut(&mut self) -> &mut [u8];
}

#[derive(PartialEq, Clone)]
pub struct OwnedBitmapData {
    pub(crate) data: Vec<u8>,
    pub(crate) width: usize,
    pub(crate) height: usize,
    pub(crate) stride: usize,
}

impl BitmapData for OwnedBitmapData {
    fn data(&self) -> &[u8] {
        self.data.as_slice()
    }
    fn width(&self) -> usize {
        self.width
    }
    fn height(&self) -> usize {
        self.height
    }
    fn stride(&self) -> usize {
        self.stride
    }
}

impl BitmapDataMut for OwnedBitmapData {
    fn data_mut(&mut self) -> &mut [u8] {
        self.data.as_mut_slice()
    }
}

#[derive(Clone)]
pub struct BorrowedBitmapData<'l> {
    pub(crate) data: &'l [u8],
    pub(crate) width: usize,
    pub(crate) height: usize,
    pub(crate) stride: usize,
}

impl<'l> BitmapData for BorrowedBitmapData<'l> {
    fn data(&self) -> &[u8] {
        self.data
    }
    fn width(&self) -> usize {
        self.width
    }
    fn height(&self) -> usize {
        self.height
    }
    fn stride(&self) -> usize {
        self.stride
    }
}

pub struct MutableBitmapData<'l> {
    pub(crate) data: &'l mut [u8],
    pub(crate) width: usize,
    pub(crate) height: usize,
    pub(crate) stride: usize,
}

impl<'l> BitmapData for MutableBitmapData<'l> {
    fn data(&self) -> &[u8] {
        self.data
    }
    fn width(&self) -> usize {
        self.width
    }
    fn height(&self) -> usize {
        self.height
    }
    fn stride(&self) -> usize {
        self.stride
    }
}

impl<'l> BitmapDataMut for MutableBitmapData<'l> {
    fn data_mut(&mut self) -> &mut [u8] {
        self.data
    }
}
